import 'dart:collection';

import 'package:panda/models/validation-config/type-config/max-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/min-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/range-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/required-validator.type-config.dart';
import 'package:panda/models/validation-config/validator-type.enum.dart';
import 'package:panda/utils/json-deserializers.dart';

class PandaValidatorConfig<TypeConfig> {
  PandaValidatorTypeEnum? type;
  String? error;
  TypeConfig? typeConfig;

  PandaValidatorConfig({
    this.error,
    this.type,
    this.typeConfig,
  });

  factory PandaValidatorConfig.fromJson(LinkedHashMap<String, dynamic> json) {
    final type = PandaValidatorTypeEnum.fromString(json["type"]);
    dynamic typeConfig;

    switch (type) {
      case PandaValidatorTypeEnum.MaxLength:
        typeConfig = PandaJsonDeserializers.deserialize<PandaMaxLengthValidatorTypeConfig>(json["typeConfig"]);
        break;
      case PandaValidatorTypeEnum.MinLength:
        typeConfig = PandaJsonDeserializers.deserialize<PandaMinLengthValidatorTypeConfig>(json["typeConfig"]);
        break;
      case PandaValidatorTypeEnum.Required:
        typeConfig = PandaJsonDeserializers.deserialize<PandaRequiredValidatorTypeConfig>(json["typeConfig"]);
        break;
      case PandaValidatorTypeEnum.RequiredIf:
        // TODO: Implement required if.
        break;
      case PandaValidatorTypeEnum.Range:
        typeConfig = PandaJsonDeserializers.deserialize<PandaRangeValidatorTypeConfig>(json["typeConfig"]);
        break;
      default:
        typeConfig = null;
    }

    return PandaValidatorConfig(
      error: json["error"],
      type: type,
      typeConfig: typeConfig,
    );
  }
}
