import 'dart:collection';

import 'package:panda/models/view-config-types/type-config.model.dart';

class PandaCheckboxTypeConfigModel implements PandaTypeConfigModel {
  late final bool? multi;
  late final List<PandaCheckboxTypeConfigModelOptionModel> options;

  PandaCheckboxTypeConfigModel({
    this.multi: false,
    required this.options,
  });

  PandaCheckboxTypeConfigModel.fromJson(LinkedHashMap<String, dynamic>? json) {
    if (json == null) {
      return;
    }

    multi = json["multi"] ?? false;
    options = List.from(json["options"]).map((optionJson) {
      return PandaCheckboxTypeConfigModelOptionModel.fromJson(optionJson);
    }).toList();
  }

  @override
  Map<String, dynamic> toJson() {
    return {
      "multi": multi,
      "options": options.map((option) => option.toJson()).toList(),
    };
  }
}

class PandaCheckboxTypeConfigModelOptionModel {
  late final int value;
  late final String text;
  late final String? description;

  PandaCheckboxTypeConfigModelOptionModel({
    required this.value,
    required this.text,
    this.description,
  });

  PandaCheckboxTypeConfigModelOptionModel.fromJson(LinkedHashMap<String, dynamic>? json) {
    if (json == null) return;

    value = json["value"];
    text = json["text"];
    description = json["description"];
  }

  Map<String, dynamic> toJson() {
    return {
      "value": value,
      "text": text,
      "description": description,
    };
  }
}
