import 'dart:collection';

import 'package:panda/models/appendable-config.model.dart';
import 'package:panda/models/validation-config/validation.model.dart';
import 'package:panda/models/view-config-types/checkbox-type-config.model.dart';
import 'package:panda/models/view-config-types/dropdown-type-config.model.dart';
import 'package:panda/models/view-config-types/group-type-config.model.dart';
import 'package:panda/models/view-config-types/short-text-type-config.model.dart';
import 'package:panda/models/view-config-types/table-type-config.model.dart';
import 'package:panda/models/view-config-types/text-type-config.model.dart';
import 'package:panda/models/view-config-types/type-config.model.dart';
import 'package:panda/models/view-config-types/view-config.type.dart';
import 'package:panda/utils/json-deserializers.dart';

class PandaViewConfigModel<TypeConfig extends PandaTypeConfigModel> {
  late String keyName;
  String? title;
  String? description;
  String? placeholder;

  // TODO: Implement display configuration.
  // DisplayConfig displayConfig;
  PandaViewConfigType? type;
  List<PandaAppendableConfigModel>? appendables;
  TypeConfig? typeConfig;
  PandaValidationConfig? validationConfig;

  PandaViewConfigModel({
    required this.keyName,
    this.title,
    this.description,
    this.placeholder,
    this.type,
    this.appendables,
    this.typeConfig,
    this.validationConfig,
  });

  factory PandaViewConfigModel.fromJson(LinkedHashMap<String, dynamic> json) {
    final type = PandaViewConfigType.fromString(json["type"]);
    dynamic typeConfig;

    switch (type) {
      case PandaViewConfigType.Checkbox:
        typeConfig = PandaJsonDeserializers.deserialize<PandaCheckboxTypeConfigModel>(json["typeConfig"]);
        break;
      case PandaViewConfigType.Dropdown:
        typeConfig = PandaJsonDeserializers.deserialize<PandaDropdownTypeConfigModel>(json["typeConfig"]);
        break;
      case PandaViewConfigType.Group:
        typeConfig = PandaJsonDeserializers.deserialize<PandaGroupTypeConfigModel>(json["typeConfig"]);
        break;
      case PandaViewConfigType.Table:
        typeConfig = PandaJsonDeserializers.deserialize<PandaTableTypeConfigModel>(json["typeConfig"]);
        break;
      case PandaViewConfigType.Text:
        typeConfig = PandaJsonDeserializers.deserialize<PandaTextTypeConfigModel>(json["typeConfig"]);
        break;
      case PandaViewConfigType.ShortText:
        typeConfig = PandaJsonDeserializers.deserialize<PandaShortTextTypeConfigModel>(json["typeConfig"]);
        break;
      default:
        typeConfig = null;
    }

    return PandaViewConfigModel(
      keyName: json["keyName"],
      title: json["title"],
      description: json["description"],
      placeholder: json["placeholder"],
      type: type,
      appendables: List.from(json["appendables"])
          .map((appendableJson) => PandaAppendableConfigModel.fromJson(appendableJson))
          .toList(),
      typeConfig: typeConfig,
      validationConfig: PandaValidationConfig.fromJson(json["validationConfig"]),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      "keyName": keyName,
      if (title != null) "title": title,
      if (description != null) "description": description,
      if (placeholder != null) "placeholder": placeholder,
      if (type != null) "type": type.toString(),
      if (appendables != null) "appendables": appendables!.map((appendable) => appendable.toJson()).toList(),
      if (typeConfig != null) "typeConfig": typeConfig!.toJson()
    };
  }

  List<String?> getChildrenConfigKeysOfType(PandaViewConfigType viewType, {String? parentKey}) {
    final currentKey = parentKey == null ? keyName : "$parentKey.$keyName";
    List<String?> keys = [];

    if (appendables != null) {
      keys.addAll(appendables!.expand((a) {
        return a.viewConfig!.getChildrenConfigKeysOfType(viewType, parentKey: currentKey);
      }).toList());
    }

    if (type == PandaViewConfigType.Group) {
      final groupConfig = typeConfig as PandaGroupTypeConfigModel;
      keys.addAll(groupConfig.viewConfigs.expand((vc) {
        return vc.getChildrenConfigKeysOfType(viewType, parentKey: currentKey);
      }).toList());
    }

    if (type == viewType) {
      keys.add(currentKey);
    }

    return keys;
  }
}
