library panda;

export 'controllers/public.dart';
export 'cubit/public.dart';
export 'enums/public.dart';
export 'layout/public.dart';
export 'models/public.dart';
export 'repositories/public.dart';
export 'utils/public.dart';
export 'widgets/public.dart';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/controllers/panda.controller.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/cubit/panda.state.dart';
import 'package:panda/enums/layout-mode.enum.dart';
import 'package:panda/layout/multi-section.layout.dart';
import 'package:panda/layout/single-section.layout.dart';
import 'package:panda/models/form-descriptor.model.dart';
import 'package:panda/models/form-result.model.dart';
import 'package:panda/repositories/panda.repository.dart';

class Panda extends StatefulWidget {
  final PandaController controller;
  final PandaFormDescriptorModel formDescriptor;
  final PandaFormResultModel? formResult;

  const Panda({
    Key? key,
    required this.controller,
    required this.formDescriptor,
    this.formResult,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _PandaState();
}

class _PandaState extends State<Panda> {
  PandaCubit? _bloc;

  PandaCubit _init() {
    _bloc = PandaCubit(RepositoryProvider.of<PandaRepository>(context));
    _bloc?.load(widget.formDescriptor, widget.formResult);

    widget.controller.bloc = _bloc;

    _bloc?.onResultChange.listen((PandaFormResultModel? result) {
      widget.controller.change(result);
    });

    _bloc?.onValidChange.listen((bool valid) {
      widget.controller.validChange(valid);
    });

    if (widget.formResult != null) {
      widget.controller.change(widget.formResult);
    }

    return _bloc!;
  }

  @override
  void dispose() {
    _bloc?.close();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      lazy: false,
      create: (_) => _init(),
      child: BlocBuilder<PandaCubit, PandaState>(
        bloc: _bloc,
        builder: (BuildContext context, PandaState state) {
          return NiceLoadingOverlay(
            loading: state.loading,
            child: state.error ? _buildPandaFailure() : _buildPandaLoaded(state),
          );
        },
      ),
    );
  }

  Widget _buildPandaLoaded(PandaState state) {
    switch (widget.controller.layoutMode) {
      case PandaLayoutModesEnum.SingleSection:
        return const PandaSingleSectionLayout();
      case PandaLayoutModesEnum.MultiSection:
        return PandaMultiSectionLayout(
          controller: widget.controller,
        );
      default:
        return Container();
    }
  }

  Widget _buildPandaFailure() {
    return Center(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Icon(
            Icons.error,
            color: Theme.of(context).errorColor,
          ),
          const SizedBox(height: 12.0),
          const Text("panda.failure"),
        ],
      ),
    );
  }
}
