import 'dart:collection';

import 'package:panda/models/validation-config/type-config/max-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/min-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/range-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/required-validator.type-config.dart';
import 'package:panda/models/view-config-types/checkbox-type-config.model.dart';
import 'package:panda/models/view-config-types/dropdown-type-config.model.dart';
import 'package:panda/models/view-config-types/group-type-config.model.dart';
import 'package:panda/models/view-config-types/short-text-type-config.model.dart';
import 'package:panda/models/view-config-types/table-type-config.model.dart';
import 'package:panda/models/view-config-types/text-type-config.model.dart';

class PandaJsonDeserializers {
  static final Map<Type, dynamic Function(LinkedHashMap<String, dynamic>)> _deserializers = {
    // ViewConfig types
    PandaCheckboxTypeConfigModel: (LinkedHashMap<String, dynamic> json) => PandaCheckboxTypeConfigModel.fromJson(json),
    PandaDropdownTypeConfigModel: (LinkedHashMap<String, dynamic> json) => PandaDropdownTypeConfigModel.fromJson(json),
    PandaGroupTypeConfigModel: (LinkedHashMap<String, dynamic> json) => PandaGroupTypeConfigModel.fromJson(json),
    PandaTableTypeConfigModel: (LinkedHashMap<String, dynamic> json) => PandaTableTypeConfigModel.fromJson(json),
    PandaTextTypeConfigModel: (LinkedHashMap<String, dynamic> json) => PandaTextTypeConfigModel.fromJson(json),
    PandaShortTextTypeConfigModel: (LinkedHashMap<String, dynamic>? json) => PandaShortTextTypeConfigModel.fromJson(json),

    // Panda Validator types
    PandaMaxLengthValidatorTypeConfig: (LinkedHashMap<String, dynamic>? json) => PandaMaxLengthValidatorTypeConfig.fromJson(json),
    PandaMinLengthValidatorTypeConfig: (LinkedHashMap<String, dynamic>? json) => PandaMinLengthValidatorTypeConfig.fromJson(json),
    PandaRequiredValidatorTypeConfig: (LinkedHashMap<String, dynamic>? json) => PandaRequiredValidatorTypeConfig.fromJson(json),
    PandaRangeValidatorTypeConfig: (LinkedHashMap<String, dynamic>? json) => PandaRangeValidatorTypeConfig.fromJson(json),
  };

  static T deserialize<T>(LinkedHashMap<String, dynamic> json) {
    if (!_deserializers.containsKey(T)) {
      throw "Unimplemented deserializer function for type ${T.toString()}";
    }

    return _deserializers[T]!(json);
  }

  static bool canDeserialize<T>() {
    return _deserializers.containsKey(T);
  }
}
