import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_exif_rotation/flutter_exif_rotation.dart';
import 'package:image_picker/image_picker.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';

class PandaPictureUtils {
  static Future<File?> takePicture(BuildContext context) async {
    final picker = ImagePicker();
    final Completer<File?> completer = Completer<File>();

    showModalBottomSheet(
      context: context,
      builder: (context) {
        return Wrap(
          children: <Widget>[
            ListTile(
              leading: const Icon(Icons.camera_alt),
              title: Text(NiceLocalizations.of(context).translate("panda.camera")),
              onTap: () async {
                Navigator.pop(context);
                completer.complete(picker.pickImage(source: ImageSource.camera, maxWidth: 1000).then(_fixRotation));
              },
            ),
            ListTile(
              leading: const Icon(Icons.image),
              title: Text(NiceLocalizations.of(context).translate("panda.gallery")),
              onTap: () {
                Navigator.pop(context);
                completer.complete(picker.pickImage(source: ImageSource.gallery, maxWidth: 1000).then(_fixRotation));
              },
            ),
          ],
        );
      },
    );

    return completer.future;
  }

  static Future<File?> _fixRotation(XFile? image) async {
    if (image?.path != null) {
      return FlutterExifRotation.rotateImage(path: image!.path);
    }

    return null;
  }
}
