import 'dart:async';
import 'dart:collection';
import 'dart:io';

import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/models/form-validation.model.dart';
import 'package:panda/models/validation-config/validation-error.model.dart';
import 'package:panda/models/view-config.model.dart';
import 'package:panda/utils/validator-config.utils.dart';
import 'package:panda/widgets/base-widget.dart';
import 'package:rxdart/rxdart.dart';

abstract class PandaBaseWidgetState<Widget extends PandaBaseWidget, ConfigType> extends State<Widget> {
  late PandaCubit _bloc;
  PandaViewConfigModel? viewConfig;

  // ignore: close_sinks
  BehaviorSubject<LinkedHashMap<String, dynamic>?>? node;

  // ignore: close_sinks
  BehaviorSubject<PandaFormValidation?>? errorNode;

  ConfigType? get typeConfig => viewConfig!.typeConfig as ConfigType?;

  bool get submitted => PandaCubit.of(context).state.submitted;

  StreamSubscription? validationSubscription;

  @override
  @mustCallSuper
  void didChangeDependencies() {
    super.didChangeDependencies();

    _bloc = BlocProvider.of<PandaCubit>(context);
    viewConfig = _bloc.getViewConfigByKeyTree(widget.keyTree);
    node = _bloc.getBehaviorSubjectFromResult(widget.keyTree);
    errorNode = _bloc.getBehaviorSubjectFromErrors(widget.keyTree);

    // VALIDATION - Each component instant listens on its model and each change in the model requires a new validation.
    validationSubscription ??= node!.listen((model) {
      final List<PandaValidationError> errors = (viewConfig?.validationConfig?.validators ?? [])
          .map((validator) => PandaValidatorConfigUtils.validate(validator, model))
          .fold([], (prev, curr) => [...prev, ...curr]);

      errorNode!.sink.add(
        PandaFormValidation(
          errors: errors,
          values: errorNode!.value?.values ?? LinkedHashMap<String, PandaFormValidation>.from({}),
        ),
      );
    });
  }

  @override
  void dispose() {
    validationSubscription?.cancel();
    super.dispose();
  }

  Future<String?> uploadFile(File file) async {
    return _bloc.uploadFile(file);
  }

  Future<String> getDownloadUrl(String fileId) {
    return _bloc.getDownloadUrl(fileId);
  }
}
