import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/models/view-config-types/checkbox-type-config.model.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';

class PandaCheckboxWidget extends PandaBaseWidget {
  const PandaCheckboxWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaCheckboxWidgetState();
}

class _PandaCheckboxWidgetState extends PandaBaseWidgetState<PandaCheckboxWidget, PandaCheckboxTypeConfigModel> {
  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Text(
          viewConfig?.title ?? NiceLocalizations.of(context).translate("panda.no_title"),
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 12.0),
          child: Wrap(
            alignment: WrapAlignment.start,
            crossAxisAlignment: WrapCrossAlignment.start,
            spacing: 12.0,
            runSpacing: 12.0,
            children: (typeConfig?.options ?? []).map((option) {
              return _buildCheckboxOrRadio(option);
            }).toList(),
          ),
        ),
      ],
    );
  }

  Widget _buildCheckboxOrRadio(PandaCheckboxTypeConfigModelOptionModel option) {
    return StreamBuilder<LinkedHashMap<String, dynamic>?>(
      stream: node!.stream,
      initialData: LinkedHashMap.from({"value": null}),
      builder: (context, AsyncSnapshot<LinkedHashMap<String, dynamic>?> snapshot) {
        if (typeConfig!.multi!) {
          final data = List<int>.from((snapshot.data ?? {})["value"] ?? []);
          return _buildCheckbox(data, option);
        } else {
          final data = (snapshot.data ?? {})["value"];
          return _buildRadio(data, option);
        }
      },
    );
  }

  Widget _buildCheckbox(List<int> checkedValues, PandaCheckboxTypeConfigModelOptionModel option) {
    return CheckboxListTile(
      title: Text(option.text),
      value: checkedValues.contains(option.value),
      onChanged: (_) => onCheck(option.value),
      controlAffinity: ListTileControlAffinity.leading,
    );
  }

  Widget _buildRadio(int? checkedValue, PandaCheckboxTypeConfigModelOptionModel option) {
    return RadioListTile(
      title: Text(option.text),
      value: option.value,
      groupValue: checkedValue,
      onChanged: onCheck,
      controlAffinity: ListTileControlAffinity.leading,
    );
  }

  void onCheck(optionValue) {
    LinkedHashMap<String, dynamic>? newViewModel = node!.valueOrNull ?? LinkedHashMap.from({"value": null});

    if (typeConfig!.multi!) {
      if (newViewModel["value"] == null) {
        newViewModel["value"] = [optionValue];
      } else if ((newViewModel["value"] as List).contains(optionValue)) {
        (newViewModel["value"] as List).remove(optionValue);
      } else {
        (newViewModel["value"] as List).add(optionValue);
      }
    } else {
      if (newViewModel["value"] == optionValue) {
        newViewModel["value"] = null;
      } else {
        newViewModel["value"] = optionValue;
      }
    }

    node!.sink.add(newViewModel);
  }
}
