import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';

class PandaCoordWidget extends PandaBaseWidget {
  const PandaCoordWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaCoordWidgetState();
}

class _PandaCoordWidgetState extends PandaBaseWidgetState<PandaCoordWidget, void> {
  final _latController = TextEditingController();
  final _lngController = TextEditingController();

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    node!.stream.listen((LinkedHashMap<String, dynamic>? data) {
      final String lat = ((data ?? const {})["value"] ?? const {})["latitude"]?.toString() ?? "";
      if (_latController.text != lat) {
        _latController.text = lat;
      }
      final String lng = ((data ?? const {})["value"] ?? const {})["longitude"]?.toString() ?? "";
      if (_lngController.text != lng) {
        _lngController.text = lng;
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Text(
          viewConfig?.title ?? NiceLocalizations.of(context).translate("panda.no_title"),
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 12.0),
          child: _buildLatitudeWidget(),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 12.0),
          child: _buildLongitudeWidget(),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 12.0),
          child: _buildDeviceLocationButton(),
        ),
      ],
    );
  }

  Widget _buildLatitudeWidget() {
    return TextFormField(
      controller: _latController,
      decoration: InputDecoration(
        border: const OutlineInputBorder(),
        labelText: NiceLocalizations.of(context).translate("panda.latitude"),
      ),
      keyboardType: const TextInputType.numberWithOptions(decimal: true, signed: true),
      onChanged: _setLatitude,
    );
  }

  Widget _buildLongitudeWidget() {
    return TextFormField(
      controller: _lngController,
      decoration: InputDecoration(
        border: const OutlineInputBorder(),
        labelText: NiceLocalizations.of(context).translate("panda.longitude"),
      ),
      keyboardType: const TextInputType.numberWithOptions(decimal: true, signed: true),
      onChanged: _setLongitude,
    );
  }

  Widget _buildDeviceLocationButton() {
    return TextButton(
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Text(
            NiceLocalizations.of(context).translate("panda.update_with_device_position"),
          ),
          const Padding(
            padding: EdgeInsets.all(5.0),
            child: Icon(Icons.gps_fixed_sharp),
          )
        ],
      ),
      onPressed: _onUpdateWithDevicePressed,
    );
  }

  void _onUpdateWithDevicePressed() {
    Geolocator.getCurrentPosition(desiredAccuracy: LocationAccuracy.best).then((Position position) {
      _setLatitude(position.latitude.toString());
      _setLongitude(position.longitude.toString());
    }).catchError((e) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(NiceLocalizations.of(context).translate("panda.could_not_get_position")),
      ));
    });
  }

  void _setLatitude(String newLatitude) {
    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull;
    if (newNodeValue == null) {
      newNodeValue = LinkedHashMap.from({"value": LinkedHashMap.from({})});
    } else if (newNodeValue["value"] == null) {
      newNodeValue["value"] = LinkedHashMap.from({});
    }

    newNodeValue["value"]["latitude"] = newLatitude;
    node!.sink.add(newNodeValue);
  }

  void _setLongitude(String newLongitude) {
    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull;
    if (newNodeValue == null) {
      newNodeValue = LinkedHashMap.from({"value": LinkedHashMap.from({})});
    } else if (newNodeValue["value"] == null) {
      newNodeValue["value"] = LinkedHashMap.from({});
    }

    newNodeValue["value"]["longitude"] = newLongitude;
    node!.sink.add(newNodeValue);
  }
}
