import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';

class PandaDateWidget extends PandaBaseWidget {
  const PandaDateWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaDateWidgetState();
}

class _PandaDateWidgetState extends PandaBaseWidgetState<PandaDateWidget, void> {
  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Text(
          viewConfig?.title ?? NiceLocalizations.of(context).translate("panda.no_title"),
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
        Padding(
          padding: const EdgeInsets.only(top: 12.0),
          child: TextButton(
            onPressed: _onPickDatePressed,
            child: Row(
              children: <Widget>[
                const Icon(Icons.calendar_today),
                Padding(
                  padding: const EdgeInsets.only(left: 12.0),
                  child: StreamBuilder<LinkedHashMap<String, dynamic>?>(
                    stream: node!.stream,
                    builder: (context, AsyncSnapshot<LinkedHashMap<String, dynamic>?> snapshot) {
                      if (snapshot.hasData && snapshot.data!["value"] != null) {
                        return Text(
                          snapshot.data!["value"],
                        );
                      }

                      return Text(
                        NiceLocalizations.of(context).translate("panda.tap_to_specify_date"),
                      );
                    },
                  ),
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }

  Future<void> _onPickDatePressed() async {
    final DateTime? picked = await showDatePicker(
      context: context,
      initialDate: DateTime.now(),
      firstDate: DateTime(0),
      lastDate: DateTime(3000),
    );

    if (picked == null) {
      return;
    }

    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull ?? LinkedHashMap.from({"value": null});

    newNodeValue["value"] = "${picked.year}/${picked.month}/${picked.day}";
    node!.sink.add(newNodeValue);
  }
}
