import 'dart:async';
import 'dart:collection';

import 'package:flutter/widgets.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/models/appendable-config.model.dart';
import 'package:panda/models/view-config-types/checkbox-type-config.model.dart';
import 'package:panda/models/view-config-types/view-config.type.dart';
import 'package:panda/models/view-config.model.dart';
import 'package:panda/widgets/checkbox.widget.dart';
import 'package:panda/widgets/coord.widget.dart';
import 'package:panda/widgets/date.widget.dart';
import 'package:panda/widgets/dropdown.widget.dart';
import 'package:panda/widgets/group.widget.dart';
import 'package:panda/widgets/long-text.widget.dart';
import 'package:panda/widgets/picture.widget.dart';
import 'package:panda/widgets/short-text.widget.dart';
import 'package:panda/widgets/time.widget.dart';

class PandaFormViewWidget extends StatefulWidget {
  final List<String> keyTree;

  const PandaFormViewWidget(this.keyTree, {Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _PandaFormViewWidgetState();
}

class _PandaFormViewWidgetState extends State<PandaFormViewWidget> {
  late PandaCubit _bloc;
  PandaViewConfigModel? _viewConfig;
  Stream<dynamic>? _model;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    _bloc = PandaCubit.of(context);
    _viewConfig = _bloc.getViewConfigByKeyTree(widget.keyTree);
    _model = _bloc.getValueObservableFromResult(widget.keyTree);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(vertical: 8.0),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          _buildMainWidgetByType(),
          ..._buildAppendables(),
        ],
      ),
    );
  }

  Widget _buildMainWidgetByType() {
    switch (_viewConfig?.type) {
      case PandaViewConfigType.Checkbox:
        return PandaCheckboxWidget(widget.keyTree);
      case PandaViewConfigType.Coord:
        return PandaCoordWidget(widget.keyTree);
      case PandaViewConfigType.Date:
        return PandaDateWidget(widget.keyTree);
      case PandaViewConfigType.Dropdown:
        return PandaDropdownWidget(widget.keyTree);
      case PandaViewConfigType.Group:
        return PandaGroupWidget(widget.keyTree);
      case PandaViewConfigType.LongText:
        return PandaLongTextWidget(widget.keyTree);
      case PandaViewConfigType.ShortText:
        return PandaShortTextWidget(widget.keyTree);
      case PandaViewConfigType.Time:
        return PandaTimeWidget(widget.keyTree);
      case PandaViewConfigType.Picture:
        return PandaPictureWidget(widget.keyTree);
      default:
        // TODO(gabriel): Implement table widget for sanitary report.
        return Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            Text(_viewConfig?.title ?? NiceLocalizations.of(context).translate("panda.no_title")),
            Text(NiceLocalizations.of(context).translate("panda.unimplemented")),
          ],
        );
    }
  }

  List<Widget> _buildAppendables() {
    return (_viewConfig?.appendables ?? [])
        .map((PandaAppendableConfigModel appendableConfig) {
          return StreamBuilder(
            stream: _model,
            initialData: null,
            builder: (BuildContext context, AsyncSnapshot<dynamic> snapshot) {
              if (shouldShowAppendable(appendableConfig, snapshot.data)) {
                return PandaFormViewWidget([...widget.keyTree, appendableConfig.viewConfig?.keyName ?? ""]);
              }

              return const SizedBox(
                width: 0,
                height: 0,
              );
            },
          );
        })
        .where((x) => x is! SizedBox)
        .toList();
  }

  bool shouldShowAppendable(PandaAppendableConfigModel config, LinkedHashMap<String, dynamic>? node) {
    if (config.expectedParentValue == null) {
      return true;
    }

    final value = (node ?? const {})["value"];
    if (value == null) {
      return false;
    }

    if (_viewConfig!.type == PandaViewConfigType.Checkbox) {
      final PandaCheckboxTypeConfigModel typeConfig = _viewConfig!.typeConfig as PandaCheckboxTypeConfigModel;
      if (typeConfig.multi!) {
        return (value as List).contains(config.expectedParentValue);
      }

      return value == config.expectedParentValue;
    } else {
      return value == config.expectedParentValue;
    }
  }
}
