import 'dart:collection';

import 'package:flutter/material.dart';
import 'package:panda/models/form-validation.model.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';

class PandaShortTextWidget extends PandaBaseWidget {
  const PandaShortTextWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaShortTextWidgetState();
}

class _PandaShortTextWidgetState extends PandaBaseWidgetState<PandaShortTextWidget, void> {
  @override
  Widget build(BuildContext context) {
    if (viewConfig == null) {
      return Container();
    }

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        if (viewConfig!.title?.isNotEmpty ?? false)
          Text(
            viewConfig!.title!,
            style: const TextStyle(fontWeight: FontWeight.bold),
          ),
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 12.0),
          child: _buildTextInput(),
        ),
        if (submitted)
          StreamBuilder<PandaFormValidation?>(
            stream: errorNode,
            builder: (_, snapshot) {
              if (snapshot.connectionState == ConnectionState.active && (snapshot.data!.errors ?? []).isNotEmpty) {
                return Text(
                  snapshot.data!.errors![0].error!,
                  style: TextStyle(
                    color: Theme.of(context).errorColor,
                  ),
                );
              }

              return Container();
            },
          ),
      ],
    );
  }

  Widget _buildTextInput() {
    return TextFormField(
      decoration: InputDecoration(
        border: const OutlineInputBorder(),
        labelText: viewConfig!.placeholder,
      ),
      initialValue: node!.value!["value"],
      onChanged: _setValue,
    );
  }

  void _setValue(String newValue) {
    LinkedHashMap<String, dynamic>? newNodeValue = node!.valueOrNull ?? LinkedHashMap.from({"value": null});

    newNodeValue["value"] = newValue;
    node!.sink.add(newNodeValue);
  }
}
